( ->
  plugin = (Vue, app, options = {}) ->

    VueVersion = Number(Vue.version and Vue.version.split('.')[0])
    initHook = if VueVersion and VueVersion > 1 then 'beforeCreate' else 'init'

    defineReactive = (vm, key, val) ->
      if key of vm
        vm[key] = val
      else
        Vue.util.defineReactive vm, key, val
      return

    Vue::$app = app
    Vue::$user = app.user
    Vue::$services = app.services

    for name, service of app.services
      if 'reading' in service.events
        Vue.util.defineReactive service, 'readingCount', service.readingCount
        Vue.util.defineReactive service, 'writingCount', service.writingCount
        Vue.util.defineReactive service, 'reading', service.reading
        Vue.util.defineReactive service, 'writing', service.writing

    Vue.mixin

      "#{initHook}": ->

        hotloads = @$options.hotloads
        hotloads = hotloads.call @ if typeof hotloads is 'function'
        for hotload, settings of hotloads
          defineReactive @, hotload, {}
        return

      created: ->

        hotloads = @$options.hotloads
        hotloads = hotloads.call @ if typeof hotloads is 'function'
        @_hotloadSubscriptions = {}
        @_hotloadUnwatchers = {}
        for hotload, settings of hotloads
          do (hotload, settings) =>
            if typeof settings isnt 'function'
              throw new Error "Hotload #{hotload} must be declared as a function."
            settings = settings.call @
            idField = settings.idField ? settings.id ? options.idField ? options.id ? '_id'
            if typeof settings.service isnt 'string'
              throw new Error "Invalid service name for hotload #{hotload}."
            service = app.service settings.service
            if typeof service.rx isnt 'function'
              throw new Error "feathers-reactive must be enabled for hotloading."
            if settings.id?
              if typeof settings.id isnt 'function'
                throw new Error "Hotload #{hotload}: id must be a function."
              @_hotloadUnwatchers[hotload] = @$watch settings.id, (id) =>
                if typeof id isnt 'string'
                  throw new Error "Invalid id for hotload #{hotload}."
                observable = service.get(id)
                @_hotloadSubscriptions[hotload]?.dispose?()
                @_hotloadSubscriptions[hotload]?.unsubscribe?()
                @_hotloadSubscriptions[hotload] = observable.subscribe (result) =>
                  @[hotload] = result
                  settings.then? result
                ,
                  (err) => settings.catch? error
              , immediate: true
            else
              settings.query ?= -> {}
              if typeof settings.query isnt 'function'
                throw new Error "Hotload #{hotload}: query must be a function."
              @_hotloadUnwatchers[hotload] = @$watch settings.query, (query) =>
                if query? and typeof query isnt 'object'
                  throw new Error "Invalid query for hotload #{hotload}."
                query ?= {}
                observable = service.find(query: query)
                @_hotloadSubscriptions[hotload]?.dispose?()
                @_hotloadSubscriptions[hotload]?.unsubscribe?()
                @_hotloadSubscriptions[hotload] = observable.subscribe (result) =>
                  isPaginated = result? and result.total? and Array.isArray result.data
                  if isPaginated
                    buffer = result.data
                  else
                    buffer = result
                  obj = {}
                  for value in buffer
                    obj[value[idField]] = value # use buffer & obj to remove duplicate items
                  @[hotload] = Object.keys obj
                  .map (key) -> obj[key]
                  if isPaginated
                    @[hotload].total = result.total
                    @[hotload].limit = result.limit
                    @[hotload].skip = result.skip
                  else
                    @[hotload].total = @[hotload].length
                    @[hotload].limit = 0
                    @[hotload].skip = 0
                  for value in @[hotload]
                    @[hotload][value[idField]] = value
                  settings.then? @[hotload]
                ,
                  (err) => settings.catch? error
              , immediate: true
        return

      beforeDestroy: ->
        for key, unwatcher of @_hotloadUnwatchers
          unwatcher()
        for key, subscription of @_hotloadSubscriptions
          subscription.dispose?()
          subscription.unsubscribe?()
        return

  if Vue? then Vue.use plugin
  if typeof exports is 'object' and typeof module is 'object'
    module.exports = plugin
  else if typeof define is 'function' and define.amd
    define -> plugin
  else if window?
    window.VueFeathersHotload = plugin

)()
