var indexOf = [].indexOf || function (item) {
	for (var i = 0, l = this.length; i < l; i++) {
		if (i in this && this[i] === item) return i;
	}
	return -1;
};

(function () {
	var plugin;
	plugin = function (Vue, app, options) {
		var VueVersion, defineReactive, initHook, name, obj1, ref, service;
		if (options == null) {
			options = {};
		}
		VueVersion = Number(Vue.version && Vue.version.split('.')[0]);
		initHook = VueVersion && VueVersion > 1 ? 'beforeCreate' : 'init';
		defineReactive = function (vm, key, val) {
			if (key in vm) {
				vm[key] = val;
			} else {
				Vue.util.defineReactive(vm, key, val);
			}
		};
		Vue.prototype.$app = app;
		Vue.prototype.$user = app.user;
		Vue.prototype.$services = app.services;
		ref = app.services;
		for (name in ref) {
			service = ref[name];
			if (indexOf.call(service.events, 'reading') >= 0) {
				Vue.util.defineReactive(service, 'readingCount', service.readingCount);
				Vue.util.defineReactive(service, 'writingCount', service.writingCount);
				Vue.util.defineReactive(service, 'reading', service.reading);
				Vue.util.defineReactive(service, 'writing', service.writing);
			}
		}
		return Vue.mixin((
			obj1 = {},
			obj1["" + initHook] = function () {
				var hotload, hotloads, settings;
				hotloads = this.$options.hotloads;
				if (typeof hotloads === 'function') {
					hotloads = hotloads.call(this);
				}
				for (hotload in hotloads) {
					settings = hotloads[hotload];
					defineReactive(this, hotload, {});
				}
			},
			obj1.created = function () {
				var fn, hotload, hotloads, settings;
				hotloads = this.$options.hotloads;
				if (typeof hotloads === 'function') {
					hotloads = hotloads.call(this);
				}
				this._hotloadSubscriptions = {};
				this._hotloadUnwatchers = {};
				fn = (function (_this) {
					return function (hotload, settings) {
						var idField, ref1, ref2, ref3, ref4;
						if (typeof settings !== 'function') {
							throw new Error("Hotload " + hotload + " must be declared as a function.");
						}
						settings = settings.call(_this);
						idField = (ref1 = (ref2 = (ref3 = (ref4 = settings.idField) != null ? ref4 : settings.id) != null ? ref3 : options.idField) != null ? ref2 : options.id) != null ? ref1 : '_id';
						if (typeof settings.service !== 'string') {
							throw new Error("Invalid service name for hotload " + hotload + ".");
						}
						service = app.service(settings.service);
						if (typeof service.rx !== 'function') {
							throw new Error("feathers-reactive must be enabled for hotloading.");
						}
						if (settings.id != null) {
							if (typeof settings.id !== 'function') {
								throw new Error("Hotload " + hotload + ": id must be a function.");
							}
							return _this._hotloadUnwatchers[hotload] = _this.$watch(settings.id, function (id) {
								var observable, ref5, ref6;
								if (typeof id !== 'string') {
									throw new Error("Invalid id for hotload " + hotload + ".");
								}
								observable = service.get(id);
								if ((ref5 = _this._hotloadSubscriptions[hotload]) != null) {
									if (typeof ref5.dispose === "function") {
										ref5.dispose();
									}
								}
								if ((ref6 = _this._hotloadSubscriptions[hotload]) != null) {
									if (typeof ref6.unsubscribe === "function") {
										ref6.unsubscribe();
									}
								}
								return _this._hotloadSubscriptions[hotload] = observable.subscribe(function (result) {
									_this[hotload] = result;
									return typeof settings.then === "function" ? settings.then(result) : void 0;
								}, function (err) {
									return typeof settings["catch"] === "function" ? settings["catch"](error) : void 0;
								});
							}, {
								immediate: true
							});
						} else {
							if (settings.query == null) {
								settings.query = function () {
									return {};
								};
							}
							if (typeof settings.query !== 'function') {
								throw new Error("Hotload " + hotload + ": query must be a function.");
							}
							return _this._hotloadUnwatchers[hotload] = _this.$watch(settings.query, function (query) {
								var observable, ref5, ref6;
								if ((query != null) && typeof query !== 'object') {
									throw new Error("Invalid query for hotload " + hotload + ".");
								}
								if (query == null) {
									query = {};
								}
								observable = service.find({
									query: query
								});
								if ((ref5 = _this._hotloadSubscriptions[hotload]) != null) {
									if (typeof ref5.dispose === "function") {
										ref5.dispose();
									}
								}
								if ((ref6 = _this._hotloadSubscriptions[hotload]) != null) {
									if (typeof ref6.unsubscribe === "function") {
										ref6.unsubscribe();
									}
								}
								return _this._hotloadSubscriptions[hotload] = observable.subscribe(function (result) {
									var buffer, i, isPaginated, j, len, len1, obj, ref7, value;
									isPaginated = (result != null) && (result.total != null) && Array.isArray(result.data);
									if (isPaginated) {
										buffer = result.data;
									} else {
										buffer = result;
									}
									obj = {};
									for (i = 0, len = buffer.length; i < len; i++) {
										value = buffer[i];
										obj[value[idField]] = value;
									}
									_this[hotload] = Object.keys(obj).map(function (key) {
										return obj[key];
									});
									if (isPaginated) {
										_this[hotload].total = result.total;
										_this[hotload].limit = result.limit;
										_this[hotload].skip = result.skip;
									} else {
										_this[hotload].total = _this[hotload].length;
										_this[hotload].limit = 0;
										_this[hotload].skip = 0;
									}
									ref7 = _this[hotload];
									for (j = 0, len1 = ref7.length; j < len1; j++) {
										value = ref7[j];
										_this[hotload][value[idField]] = value;
									}
									return typeof settings.then === "function" ? settings.then(_this[hotload]) : void 0;
								}, function (err) {
									return typeof settings["catch"] === "function" ? settings["catch"](error) : void 0;
								});
							}, {
								immediate: true
							});
						}
					};
				})(this);
				for (hotload in hotloads) {
					settings = hotloads[hotload];
					fn(hotload, settings);
				}
			},
			obj1.beforeDestroy = function () {
				var key, ref1, ref2, subscription, unwatcher;
				ref1 = this._hotloadUnwatchers;
				for (key in ref1) {
					unwatcher = ref1[key];
					unwatcher();
				}
				ref2 = this._hotloadSubscriptions;
				for (key in ref2) {
					subscription = ref2[key];
					if (typeof subscription.dispose === "function") {
						subscription.dispose();
					}
					if (typeof subscription.unsubscribe === "function") {
						subscription.unsubscribe();
					}
				}
			},
			obj1
		));
	};
	if (typeof Vue !== "undefined" && Vue !== null) {
		Vue.use(plugin);
	}
	if (typeof exports === 'object' && typeof module === 'object') {
		return module.exports = plugin;
	} else if (typeof define === 'function' && define.amd) {
		return define(function () {
			return plugin;
		});
	} else if (typeof window !== "undefined" && window !== null) {
		return window.VueFeathersHotload = plugin;
	}
})();